--
-- Copyright (C) 2012-2016 Snail <https://github.com/snail23/snailui/>
--
-- This program is free software; you can redistribute it and/or modify it
-- under the terms of the GNU General Public License as published by the
-- Free Software Foundation; either version 2 of the License, or (at your
-- option) any later version.
--
-- This program is distributed in the hope that it will be useful, but WITHOUT
-- ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-- FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
-- more details.
--
-- You should have received a copy of the GNU General Public License along
-- with this program. If not, see <http://www.gnu.org/licenses/>.
--

local _
local Index
local Nameplates

local function HandleHealer(Self)
	local CastingSpell = Self.ArtContainer.CastBarText:GetText()
	local Name = Self.NameContainer.NameText:GetText()

	if Name then
		Name = Name:gsub(" [(*)]+", "")

		if (Self.Name ~= Name) or (CastingSpell and Self.ArtContainer.CastBar:IsShown() and not Self.VC) then
			if Self.Name ~= Name then
				Self.VC = nil
			end

			Self.Healer = nil
			Self.Name = Name

			local DDSpells =
			{
				-- Druid: Balance

				"Starfall",
				"Starfire",
				"Starsurge",

				-- Druid: Feral

				"Rake",
				"Rip",
				"Swipe",

				-- Druid: Guardian

				"Maul",
				"Lacerate",

				-- Monk: Brewmaster

				"Breath of Fire",
				"Keg Smash",

				-- Monk: Windwalker

				"Fists of Fury",
				"Touch of Karma",

				-- Paladin: Protection

				"Avenger's Shield",
				"Hammer of the Righteous",
				"Judgment",

				-- Paladin: Retribution

				"Censure",
				"Divine Storm",
				"Exorcism",
				"Shield of the Righteous",
				"Templar's Verdict",

				-- Priest: Shadow

				"Devouring Plague",
				"Mind Blast",
				"Mind Flay",
				"Mind Spike",

				-- Shaman: Elemental

				"Earth Shock",
				"Earthquake",
				"Fulmination",
				"Molten Earth",

				-- Shaman: Enhancement

				"Lava Lash",
				"Stormstrike",
				"Windfury",
			}

			local HealerClasses =
			{
				"DRUID",
				"MONK",
				"PALADIN",
				"PRIEST",
				"SHAMAN"
			}

			local HealerSpells =
			{
				-- Druid

				"Genesis",
				"Ironbark",
				"Lifebloom",
				"Regrowth",
				"Swiftmend",
				"Wild Growth",

				-- Monk

				"Enveloping Mist",
				"Life Cocoon",
				"Renewing Mist",
				"Soothing Mist",
				"Uplift",
				
				-- Paladin

				"Beacon of Light",
				"Holy Radiance",
				"Holy Shock",
				"Illuminated Healing",
				"Light of Dawn",

				-- Priest: Discipline

				"Clarity of Will",
				"Divine Aegis",
				"Pain Suppression",
				"Penance",
				"Spirit Shell",

				-- Priest: Holy

				"Circle of Healing",
				"Clarity of Purpose",
				"Divine Hymn",
				"Echo of Light",
				"Guardian Spirit",
				"Lightwell",
				"Renew",

				-- Shaman

				"Chain Heal",
				"Earth Shield",
				"Healing Tide Totem",
				"Riptide",
				"Spirit Link Totem"
			}

			local DD = true

			for _, HealerSpell in ipairs(HealerSpells) do
				for _, Fight in ipairs(Data.Meter.Fights) do
					for _, Player in ipairs(Fight) do
						if Self.Name == Player.Name then
							for _, HealerClass in ipairs(HealerClasses) do
								if Player.Class == HealerClass then
									DD = nil

									break
								end
							end

							if not DD and not Self.Healer then
								for _, Spell in ipairs(Player.HealingSpells) do
									if Spell.SpellName == HealerSpell then
										Self.Healer = true

										break
									end
								end
							end

							if not DD and not Self.Healer then
								for _, Spell in ipairs(Player.DamageSpells) do
									for _, DDSpell in ipairs(DDSpells) do
										if Spell.SpellName == DDSpell then
											DD = true

											break
										end
									end

									if not DD then
										if Spell.SpellName == HealerSpell then
											Self.Healer = true

											break
										end
									else
										break
									end
								end
							else
								break
							end
						end
					end

					if Self.Healer then
						break
					end
				end

				if Self.Healer then
					break
				end
			end

			for _, HealerSpell in ipairs(HealerSpells) do
				if (CastingSpell == HealerSpell) and Self.ArtContainer.CastBar:IsShown() and not DD and not Self.VC then
					Self.ArtContainer.CastBarText:SetText()
					Self.Healer = true
					Self.VC = true

					break
				end
			end

			if Self.Healer then
				Self.HealerIcon:Show()
			else
				Self.HealerIcon:Hide()
			end
		end
	end
end

local function HandleNameplate(Self)
	local Frames =
	{
		"AbsorbBar",
		"AggroWarningTexture",
		"Border",
		"CastBar",
		"EliteIcon",
		"HealthBar",
		"HighLevelIcon",
		"Highlight",
		"LevelText",
		"RaidTargetIcon"
	}

	for I = 1, #Frames do
		Self.ArtContainer[Frames[I]]:Hide()
		Self.ArtContainer[Frames[I]]:SetParent(Nameplates.Trash)
	end

	local function UpdateNameplate(Self)
		Self:SetScale(0.75)

		if GetConfiguration().Nameplates.HealthBar then
			Self.Health:SetValue(Self.ArtContainer.HealthBar:GetValue())

			if GetConfiguration().Nameplates.HealthBar.Name then
				Self.Health.Name:SetText(Self.NameContainer.NameText:GetText())
			end
		end

		if GetConfiguration().Nameplates.HealerIcon then
			HandleHealer(Self)
		end

		if Self.ArtContainer.AggroWarningTexture:IsShown() then
			local AR, AG, AB = Self.ArtContainer.AggroWarningTexture:GetVertexColor()

			if (AR > 0.9) and (AG == 0) and (AB == 0) then
				Self.Background:SetTexture(1, 0, 0.75)

				if GetConfiguration().Nameplates.HealerIcon then
					Self.HealerIcon:SetVertexColor(1, 0, 0.75)
				end

				if GetConfiguration().Nameplates.HealthBar then
					Self.Health:SetStatusBarColor(1, 0, 0.75)
					Self.Health.Name:SetTextColor(Configuration.TextColorPercentage, 0, 0.75 * Configuration.TextColorPercentage)
				end
			else 
				Self.Background:SetTexture(AR, AG, AB)

				if GetConfiguration().Nameplates.HealerIcon then
					Self.HealerIcon:SetVertexColor(AR, AG, AB)
				end

				if GetConfiguration().Nameplates.HealthBar then
					Self.Health:SetStatusBarColor(AR, AG, AB)
					Self.Health.Name:SetTextColor(AR * Configuration.TextColorPercentage, AG * Configuration.TextColorPercentage, AB * Configuration.TextColorPercentage)
				end
			end
		else
			local R, G, B = Self.ArtContainer.HealthBar:GetStatusBarColor()

			Self.Background:SetTexture(R, G, B)

			if GetConfiguration().Nameplates.HealerIcon then
				Self.HealerIcon:SetVertexColor(R, G, B)
			end

			if GetConfiguration().Nameplates.HealthBar then
				Self.Health:SetStatusBarColor(R, G, B)
				Self.Health.Name:SetTextColor(R * Configuration.TextColorPercentage, G * Configuration.TextColorPercentage, B * Configuration.TextColorPercentage)
			end
		end
	end

	Self:SetHeight(GetConfiguration().Nameplates.Height)
	Self:SetScale(0.75)
	Self:SetScript("OnUpdate",
		function(Self, ElapsedTime)
			if not Self.Time then
				Self.Time = 0
			end

			if (Self.Time + ElapsedTime) >= 0.1 then
				UpdateNameplate(Self)
				Self.Time = 0
			else
				Self.Time = Self.Time + ElapsedTime
			end
		end
	)

	Self:SetScript("OnShow",
		function(Self)
			if GetConfiguration().Nameplates.HealerIcon then
				HandleHealer(Self)
			end

			if GetConfiguration().Nameplates.HealthBar then
				Self.Health:SetMinMaxValues(Self.ArtContainer.HealthBar:GetMinMaxValues())
			end

			UpdateNameplate(Self)
		end
	)

	Self.ArtContainer:Hide()
	Self.ArtContainer:SetParent(Nameplates.Trash)
	Self.ArtContainer.HealthBar:SetScript("OnValueChanged",
		function(_, Value)
			if GetConfiguration().Nameplates.HealthBar then
				Self.Health:SetValue(Value)
			end
		end
	)

	Self.Background = Self:CreateTexture(nil, "LOW")
	Self.Background:SetPoint("CENTER")
	Self.Background:SetSize(math.floor(Self:GetWidth() + 0.5) - 2, GetConfiguration().Nameplates.Height - 2)

	Self.Border = Self:CreateTexture(nil, "BACKGROUND")
	Self.Border:SetPoint("CENTER")
	Self.Border:SetSize(math.floor(Self:GetWidth() + 0.5), GetConfiguration().Nameplates.Height)
	Self.Border:SetTexture(0, 0, 0)

	if GetConfiguration().Nameplates.HealerIcon then
		Self.HealerIcon = Self:CreateTexture(nil, "LOW")
		Self.HealerIcon:Hide()
		Self.HealerIcon:SetPoint(GetConfiguration().Nameplates.HealerIcon.Anchor, GetConfiguration().Nameplates.HealerIcon.X, GetConfiguration().Nameplates.HealerIcon.Y)
		Self.HealerIcon:SetTexture(Configuration.HealerTexture)
	end

	if GetConfiguration().Nameplates.HealthBar then
		Self.Health = CreateFrame("StatusBar", nil, Self)
		Self.Health:SetOrientation(GetConfiguration().Nameplates.HealthBar.Orientation)
		Self.Health:SetPoint(GetConfiguration().Nameplates.HealthBar.Anchor, GetConfiguration().Nameplates.HealthBar.X, GetConfiguration().Nameplates.HealthBar.Y)
		Self.Health:SetSize(math.floor(Self:GetWidth() + 0.5) - 6, GetConfiguration().Nameplates.HealthBar.Height - 2)
		Self.Health:SetStatusBarTexture(Configuration.Texture)
		
		SmoothBar(Self, Self.Health)

		Self.Health.Border = Self.Health:CreateTexture(nil, "LOW")
		Self.Health.Border:SetPoint("CENTER")
		Self.Health.Border:SetSize(math.floor(Self:GetWidth() + 0.5) - 4, GetConfiguration().Nameplates.HealthBar.Height)
		Self.Health.Border:SetTexture(0, 0, 0)

		if GetConfiguration().Nameplates.HealthBar.Name then
			Self.Health.Name = Self.Health:CreateFontString(nil, "OVERLAY")
			Self.Health.Name:SetFont(Configuration.Font.Name, Configuration.Font.Size, Configuration.Font.Outline)
			Self.Health.Name:SetPoint(GetConfiguration().Nameplates.HealthBar.Name.Anchor, GetConfiguration().Nameplates.HealthBar.Name.X, GetConfiguration().Nameplates.HealthBar.Name.Y)
		end
	end

	Self.NameContainer:Hide()
	Self.NameContainer:SetParent(Nameplates.Trash)

	Self.Skinned = true
	Index = Index + 1
end

function HandleNameplates()
	if GetConfiguration().Nameplates then
		Nameplates = CreateFrame("Frame", nil, UIParent)
		Nameplates:SetScript("OnUpdate",
			function(Self, ElapsedTime)
				if not Self.Time then
					Self.Time = 0
				end

				if (Self.Time + ElapsedTime) >= 0.1 then
					local Nameplate

					if not Index then
						for _, Nameplate in next, {WorldFrame:GetChildren()} do
							local Name = Nameplate:GetName()

							if Name and string.match(Name, "^NamePlate%d+$") and not Nameplate.Skinned then
								Index = string.match(Name, "(%d+)")
								HandleNameplate(Nameplate)
							end
						end
					else
						Nameplate = _G["NamePlate" .. Index]

						if Nameplate and not Nameplate.Skinned then
							HandleNameplate(Nameplate)
						end
					end

					Self.Time = 0
				else
					Self.Time = Self.Time + ElapsedTime
				end
			end
		)

		Nameplates.Trash = CreateFrame("Frame")
		Nameplates.Trash:Hide()
	end
end
